@@ -58,17 +58,13 @@ enum {
   ARG_FRAME_TIME,
 };
 
-GST_PAD_TEMPLATE_FACTORY (sink_template,
+static GstStaticPadTemplate sink_template =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  gst_caps_new (
-    "aasink_caps",
-    "video/x-raw-yuv",
-      GST_VIDEO_YUV_PAD_TEMPLATE_PROPS (
-	      GST_PROPS_FOURCC (GST_STR_FOURCC ("I420")))
-  )
-)
+  GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("I420"))
+);
 
 static void	gst_aasink_base_init	(gpointer g_class);
 static void	gst_aasink_class_init	(GstAASinkClass *klass);
@@ -177,7 +173,7 @@ gst_aasink_base_init (gpointer g_class)
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
   gst_element_class_add_pad_template (element_class,
-		GST_PAD_TEMPLATE_GET (sink_template));
+      gst_static_pad_template_get (&sink_template));
   gst_element_class_set_details (element_class, &gst_aasink_details);
 }
 
@@ -230,11 +226,11 @@ gst_aasink_class_init (GstAASinkClass *klass)
   gobject_class->get_property = gst_aasink_get_property;
 
   gst_aasink_signals[SIGNAL_FRAME_DISPLAYED] =
-    g_signal_new ("frame_displayed", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
+    g_signal_new ("frame-displayed", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
                    G_STRUCT_OFFSET (GstAASinkClass, frame_displayed), NULL, NULL,
                    g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
   gst_aasink_signals[SIGNAL_HAVE_SIZE] =
-    g_signal_new ("have_size", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
+    g_signal_new ("have-size", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
                    G_STRUCT_OFFSET (GstAASinkClass, have_size), NULL, NULL,
                    gst_marshal_VOID__INT_INT, G_TYPE_NONE, 2,
                    G_TYPE_UINT, G_TYPE_UINT);
@@ -243,18 +239,43 @@ gst_aasink_class_init (GstAASinkClass *klass)
   gstelement_class->set_clock    = gst_aasink_set_clock;
 }
 
+static GstCaps *
+gst_aasink_fixate (GstPad *pad, const GstCaps *caps)
+{
+  GstStructure *structure;
+  GstCaps *newcaps;
+
+  if (gst_caps_get_size (caps) > 1) return NULL;
+
+  newcaps = gst_caps_copy (caps);
+  structure = gst_caps_get_structure (newcaps, 0);
+
+  if (gst_caps_structure_fixate_field_nearest_int (structure, "width", 320)) {
+    return newcaps;
+  }
+  if (gst_caps_structure_fixate_field_nearest_int (structure, "height", 240)) {
+    return newcaps;
+  }
+  if (gst_caps_structure_fixate_field_nearest_double (structure, "framerate",
+                                                      30.0)) {
+    return newcaps;
+  }
+
+  gst_caps_free (newcaps);
+  return NULL;
+}
+
 static GstPadLinkReturn
-gst_aasink_sinkconnect (GstPad *pad, GstCaps *caps)
+gst_aasink_sinkconnect (GstPad *pad, const GstCaps *caps)
 {
   GstAASink *aasink;
+  GstStructure *structure;
 
   aasink = GST_AASINK (gst_pad_get_parent (pad));
 
-  if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_LINK_DELAYED;
-  
-  gst_caps_get_int (caps, "width", &aasink->width);
-  gst_caps_get_int (caps, "height", &aasink->height);
+  structure = gst_caps_get_structure (caps, 0);
+  gst_structure_get_int (structure, "width", &aasink->width);
+  gst_structure_get_int (structure, "height", &aasink->height);
 
   /* FIXME aasink->format is never set */
 
@@ -279,10 +300,11 @@ static void
 gst_aasink_init (GstAASink *aasink)
 {
   aasink->sinkpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (sink_template), "sink");
+      gst_element_get_pad_template (GST_ELEMENT (aasink), "sink"), "sink");
   gst_element_add_pad (GST_ELEMENT (aasink), aasink->sinkpad);
   gst_pad_set_chain_function (aasink->sinkpad, gst_aasink_chain);
   gst_pad_set_link_function (aasink->sinkpad, gst_aasink_sinkconnect);
+  gst_pad_set_fixate_function (aasink->sinkpad, gst_aasink_fixate);
 
   memcpy(&aasink->ascii_surf, &aa_defparams, sizeof (struct aa_hardware_params));
   aasink->ascii_parms.bright = 0;
@@ -361,10 +383,8 @@ gst_aasink_chain (GstPad *pad, GstData *_data)
 
   GST_DEBUG ("videosink: clock wait: %" G_GUINT64_FORMAT, GST_BUFFER_TIMESTAMP(buf));
 
-  if (aasink->clock) {
-    GstClockID id = gst_clock_new_single_shot_id (aasink->clock, GST_BUFFER_TIMESTAMP(buf));
-    gst_element_clock_wait (GST_ELEMENT (aasink), id, NULL);
-    gst_clock_id_free (id);
+  if (aasink->clock && GST_BUFFER_TIMESTAMP_IS_VALID (buf)) {
+    gst_element_wait (GST_ELEMENT (aasink), GST_BUFFER_TIMESTAMP (buf));
   }
 
   aa_render (aasink->context, &aasink->ascii_parms, 
@@ -497,8 +517,8 @@ gst_aasink_open (GstAASink *aasink)
 
   aasink->context = aa_autoinit (&aasink->ascii_surf);
   if (aasink->context == NULL) {
-    gst_element_error (GST_ELEMENT (aasink), 
-		    g_strdup("opening aalib context"));
+    GST_ELEMENT_ERROR (GST_ELEMENT (aasink), LIBRARY, TOO_LAZY, (NULL),
+		         ("error opening aalib context"));
     return FALSE;
   }
   aa_autoinitkbd(aasink->context, 0);
@@ -557,7 +577,6 @@ GST_PLUGIN_DEFINE (
   plugin_init,
   VERSION,
   "GPL",
-  GST_COPYRIGHT,
   GST_PACKAGE,
   GST_ORIGIN
 )
