@@ -18,21 +18,19 @@
  */
 
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 
 #include "gstgsmdec.h"
 
-extern GstPadTemplate *gsmdec_src_template, *gsmdec_sink_template;
-
 /* elementfactory information */
 GstElementDetails gst_gsmdec_details = {
-  "gsm audio decoder",
-  "Codec/Audio/Decoder",
-  "LGPL",
-  ".gsm",
-  VERSION,
+  "GSM audio decoder",
+  "Codec/Decoder/Audio",
+  "Decodes GSM encoded audio",
   "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 2000",
 };
 
 /* GSMDec signals and args */
@@ -46,11 +44,12 @@ enum {
   /* FILL ME */
 };
 
+static void                     gst_gsmdec_base_init (gpointer g_class);
 static void			gst_gsmdec_class_init	(GstGSMDec *klass);
 static void			gst_gsmdec_init		(GstGSMDec *gsmdec);
 
-static void			gst_gsmdec_chain	(GstPad *pad, GstBuffer *buf);
-static GstPadLinkReturn	gst_gsmdec_sinkconnect 	(GstPad *pad, GstCaps *caps);
+static void			gst_gsmdec_chain	(GstPad *pad, GstData *_data);
+static GstPadLinkReturn	gst_gsmdec_sinkconnect 	(GstPad *pad, const GstCaps *caps);
 
 static GstElementClass *parent_class = NULL;
 /*static guint gst_gsmdec_signals[LAST_SIGNAL] = { 0 }; */
@@ -61,7 +60,8 @@ gst_gsmdec_get_type(void) {
 
   if (!gsmdec_type) {
     static const GTypeInfo gsmdec_info = {
-      sizeof(GstGSMDecClass),      NULL,
+      sizeof(GstGSMDecClass),
+      gst_gsmdec_base_init,
       NULL,
       (GClassInitFunc)gst_gsmdec_class_init,
       NULL,
@@ -75,6 +75,46 @@ gst_gsmdec_get_type(void) {
   return gsmdec_type;
 }
 
+static GstStaticPadTemplate gsmdec_sink_template =
+GST_STATIC_PAD_TEMPLATE (
+    "sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS (
+      "audio/x-gsm, "
+      "rate = (int) [ 1000, 48000 ], "
+      "channels = (int) 1"
+    )
+);
+
+static GstStaticPadTemplate gsmdec_src_template =
+GST_STATIC_PAD_TEMPLATE (
+    "src",
+    GST_PAD_SRC,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS (
+      "audio/x-raw-int, "
+      "endianness = (int) BYTE_ORDER, "
+      "signed = (boolean) true, "
+      "width = (int) 16, "
+      "depth = (int) 16, "
+      "rate = (int) [ 1000, 48000 ], "
+      "channels = (int) 1"
+  )
+);
+
+static void
+gst_gsmdec_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gsmdec_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gsmdec_src_template));
+  gst_element_class_set_details (element_class, &gst_gsmdec_details);
+}
+
 static void
 gst_gsmdec_class_init (GstGSMDec *klass)
 {
@@ -88,15 +128,17 @@ gst_gsmdec_class_init (GstGSMDec *klass)
 static void
 gst_gsmdec_init (GstGSMDec *gsmdec)
 {
-  GST_DEBUG (0,"gst_gsmdec_init: initializing");
+  GST_DEBUG ("gst_gsmdec_init: initializing");
 
   /* create the sink and src pads */
-  gsmdec->sinkpad = gst_pad_new_from_template (gsmdec_sink_template, "sink");
+  gsmdec->sinkpad = gst_pad_new_from_template (
+      gst_static_pad_template_get (&gsmdec_sink_template), "sink");
   gst_element_add_pad (GST_ELEMENT (gsmdec), gsmdec->sinkpad);
   gst_pad_set_chain_function (gsmdec->sinkpad, gst_gsmdec_chain);
   gst_pad_set_link_function (gsmdec->sinkpad, gst_gsmdec_sinkconnect);
 
-  gsmdec->srcpad = gst_pad_new_from_template (gsmdec_src_template, "src");
+  gsmdec->srcpad = gst_pad_new_from_template (
+      gst_static_pad_template_get (&gsmdec_src_template), "src");
   gst_element_add_pad (GST_ELEMENT (gsmdec), gsmdec->srcpad);
 
   gsmdec->state = gsm_create ();
@@ -104,31 +146,26 @@ gst_gsmdec_init (GstGSMDec *gsmdec)
 }
 
 static GstPadLinkReturn
-gst_gsmdec_sinkconnect (GstPad *pad, GstCaps *caps)
+gst_gsmdec_sinkconnect (GstPad *pad, const GstCaps *caps)
 {
   GstGSMDec *gsmdec;
   gint rate;
+  GstStructure *structure;
   
   gsmdec = GST_GSMDEC (gst_pad_get_parent (pad));
 
-  if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_LINK_DELAYED;
-  
-  gst_caps_get_int (caps, "rate", &rate);
+  structure = gst_caps_get_structure (caps, 0);
+  gst_structure_get_int  (structure, "rate", &rate);
 
   if (gst_pad_try_set_caps (gsmdec->srcpad, 
-		      GST_CAPS_NEW (
-	  		"gsm_raw",
-			"audio/raw",
-			  "format",       GST_PROPS_STRING ("int"),
-			    "law",        GST_PROPS_INT (0),
-			    "endianness", GST_PROPS_INT (G_BYTE_ORDER),
-			    "signed",     GST_PROPS_BOOLEAN (TRUE),
-			    "width",      GST_PROPS_INT (16),
-			    "depth",      GST_PROPS_INT (16),
-			    "rate",       GST_PROPS_INT (rate),
-			    "channels",   GST_PROPS_INT (1)
-			   )) > 0)
+        gst_caps_new_simple ("audio/x-raw-int",
+          "endianness", G_TYPE_INT, G_BYTE_ORDER,
+          "signed",     G_TYPE_BOOLEAN, TRUE,
+          "width",      G_TYPE_INT, 16,
+          "depth",      G_TYPE_INT, 16,
+          "rate",       G_TYPE_INT, rate,
+          "channels",   G_TYPE_INT, 1,
+          NULL)) > 0)
   {
     return GST_PAD_LINK_OK;
   }
@@ -136,8 +173,9 @@ gst_gsmdec_sinkconnect (GstPad *pad, GstCaps *caps)
 }
 
 static void
-gst_gsmdec_chain (GstPad *pad, GstBuffer *buf)
+gst_gsmdec_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstGSMDec *gsmdec;
   gsm_byte *data;
   guint size;
@@ -163,7 +201,7 @@ gst_gsmdec_chain (GstPad *pad, GstBuffer *buf)
 
     gsm_decode (gsmdec->state, gsmdec->buffer, (gsm_signal *) GST_BUFFER_DATA (outbuf));
 
-    gst_pad_push (gsmdec->srcpad, outbuf);
+    gst_pad_push (gsmdec->srcpad, GST_DATA (outbuf));
 
     size -= (33 - gsmdec->bufsize);
     data += (33 - gsmdec->bufsize);
@@ -179,7 +217,7 @@ gst_gsmdec_chain (GstPad *pad, GstBuffer *buf)
     
     gsm_decode (gsmdec->state, data, (gsm_signal *)GST_BUFFER_DATA (outbuf));
 
-    gst_pad_push (gsmdec->srcpad, outbuf);
+    gst_pad_push (gsmdec->srcpad, GST_DATA (outbuf));
 
     size -= 33;
     data += 33;
