@@ -25,7 +25,7 @@
 #endif
 
 #include "colorbalance.h"
-#include "colorbalancemarshal.h"
+#include "colorbalance-marshal.h"
 
 enum {
   VALUE_CHANGED,
@@ -71,7 +71,7 @@ gst_color_balance_class_init (GstColorBalanceClass *klass)
   
   if (!initialized) {
     gst_color_balance_signals[VALUE_CHANGED] =
-      g_signal_new ("value_changed",
+      g_signal_new ("value-changed",
 		    GST_TYPE_COLOR_BALANCE, G_SIGNAL_RUN_LAST,
 		    G_STRUCT_OFFSET (GstColorBalanceClass, value_changed),
 		    NULL, NULL,
@@ -82,6 +82,8 @@ gst_color_balance_class_init (GstColorBalanceClass *klass)
     initialized = TRUE;
   }
 
+  klass->balance_type = GST_COLOR_BALANCE_SOFTWARE;
+  
   /* default virtual functions */
   klass->list_channels = NULL;
   klass->set_value = NULL;
