@@ -19,6 +19,9 @@
  */
 
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 
 #include "gstcdxaparse.h"
@@ -40,21 +43,8 @@
 static GstElementDetails gst_cdxa_parse_details = {
   ".dat parser",
   "Codec/Parser",
-  "LGPL",
   "Parse a .dat file (VCD) into raw mpeg1",
-  VERSION,
   "Wim Taymans <wim.taymans@tvd.be>",
-  "(C) 2002",
-};
-
-static GstCaps* cdxa_type_find (GstBuffer *buf, gpointer private);
-
-/* typefactory for 'cdxa' */
-static GstTypeDefinition cdxadefinition = {
-  "cdxaparse_video/avi",
-  "video/avi",
-  ".dat",
-  cdxa_type_find,
 };
 
 /* CDXAParse signals and args */
@@ -68,29 +58,25 @@ enum {
   /* FILL ME */
 };
 
-GST_PAD_TEMPLATE_FACTORY (sink_templ,
+static GstStaticPadTemplate sink_templ =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "cdxaparse_sink",
-     "video/avi",
-      "format", GST_PROPS_STRING ("CDXA")
-  )
-)
+  GST_STATIC_CAPS ( "video/x-cdxa" )
+);
 
-GST_PAD_TEMPLATE_FACTORY (src_templ,
+static GstStaticPadTemplate src_templ =
+GST_STATIC_PAD_TEMPLATE (
   "src",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "cdxaparse_src",
-    "video/mpeg",
-      "mpegversion",   GST_PROPS_INT (1),
-      "systemstream",  GST_PROPS_BOOLEAN (TRUE)
+  GST_STATIC_CAPS ( "video/mpeg, "
+      "systemstream = (boolean) TRUE"
   )
-)
+);
 
+static void     gst_cdxa_parse_base_init        (gpointer g_class);
 static void 	gst_cdxa_parse_class_init	(GstCDXAParseClass *klass);
 static void 	gst_cdxa_parse_init		(GstCDXAParse *cdxa_parse);
 
@@ -111,7 +97,7 @@ gst_cdxa_parse_get_type(void)
   if (!cdxa_parse_type) {
     static const GTypeInfo cdxa_parse_info = {
       sizeof(GstCDXAParseClass),      
-      NULL,
+      gst_cdxa_parse_base_init,
       NULL,
       (GClassInitFunc)gst_cdxa_parse_class_init,
       NULL,
@@ -125,6 +111,18 @@ gst_cdxa_parse_get_type(void)
   return cdxa_parse_type;
 }
 
+static void
+gst_cdxa_parse_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_templ));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_templ));
+  gst_element_class_set_details (element_class, &gst_cdxa_parse_details);
+}
+
 static void
 gst_cdxa_parse_class_init (GstCDXAParseClass *klass) 
 {
@@ -145,38 +143,17 @@ gst_cdxa_parse_init (GstCDXAParse *cdxa_parse)
   GST_FLAG_SET (cdxa_parse, GST_ELEMENT_EVENT_AWARE);
 				
   cdxa_parse->sinkpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (sink_templ), "sink");
+		  gst_static_pad_template_get (&sink_templ), "sink");
   gst_element_add_pad (GST_ELEMENT (cdxa_parse), cdxa_parse->sinkpad);
 
   cdxa_parse->srcpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (src_templ), "src");
+		  gst_static_pad_template_get (&src_templ), "src");
   gst_element_add_pad (GST_ELEMENT (cdxa_parse), cdxa_parse->srcpad);
 
   gst_element_set_loop_function (GST_ELEMENT (cdxa_parse), gst_cdxa_parse_loop);
 
 }
 
-static GstCaps*
-cdxa_type_find (GstBuffer *buf,
-              gpointer private)
-{
-  gchar *data = GST_BUFFER_DATA (buf);
-  GstCaps *new;
-
-  GST_DEBUG (0,"cdxa_parse: typefind");
-
-  if (GUINT32_FROM_LE (((guint32 *)data)[0]) != GST_RIFF_TAG_RIFF)
-    return NULL;
-  if (GUINT32_FROM_LE (((guint32 *)data)[2]) != GST_RIFF_RIFF_CDXA)
-    return NULL;
-
-  new = GST_CAPS_NEW ("cdxa_type_find",
-		      "video/avi", 
-		        "RIFF", GST_PROPS_STRING ("CDXA"));
-
-  return new;
-}
-
 static gboolean
 gst_cdxa_parse_handle_event (GstCDXAParse *cdxa_parse)
 {
@@ -297,7 +274,7 @@ gst_cdxa_parse_loop (GstElement *element)
     outbuf = gst_buffer_create_sub (buf, 24, CDXA_DATA_SIZE);
     gst_buffer_unref (buf);
 
-    gst_pad_push (cdxa_parse->srcpad, outbuf);
+    gst_pad_push (cdxa_parse->srcpad, GST_DATA (outbuf));
   }
 }
 
@@ -332,35 +309,25 @@ gst_cdxa_parse_change_state (GstElement *element)
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-  GstTypeFactory *type;
-
-  /* this filter needs the riff parser */
   if (!gst_library_load ("gstbytestream"))
     return FALSE;
 
-  /* create an elementfactory for the cdxa_parse element */
-  factory = gst_element_factory_new ("cdxaparse", GST_TYPE_CDXA_PARSE,
-                                    &gst_cdxa_parse_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (src_templ));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (sink_templ));
-
-  type = gst_type_factory_new (&cdxadefinition);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (type));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
+  if (!gst_element_register (plugin, "cdxaparse", GST_RANK_NONE, GST_TYPE_CDXA_PARSE))
+    return FALSE;
+  
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "cdxaparse",
-  plugin_init
-};
+  "Parse a .dat file (VCD) into raw mpeg1",  
+  plugin_init,
+  VERSION,
+  "LGPL",
+  GST_PACKAGE,
+  GST_ORIGIN)
 
