@@ -28,15 +28,12 @@
 #define UDP_DEFAULT_CONTROL	1
 
 /* elementfactory information */
-GstElementDetails gst_udpsink_details = {
+static GstElementDetails gst_udpsink_details = GST_ELEMENT_DETAILS (
   "UDP packet sender",
   "Sink/Network",
-  "LGPL",
   "Send data over the network via UDP",
-  VERSION,
-  "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 2001",
-};
+  "Wim Taymans <wim.taymans@chello.be>"
+);
 
 /* UDPSink signals and args */
 enum {
@@ -70,6 +67,7 @@ gst_udpsink_control_get_type(void) {
   return udpsink_control_type;
 }
 
+static void		gst_udpsink_base_init		(gpointer g_class);
 static void		gst_udpsink_class_init		(GstUDPSink *klass);
 static void		gst_udpsink_init		(GstUDPSink *udpsink);
 
@@ -92,11 +90,10 @@ gst_udpsink_get_type (void)
 {
   static GType udpsink_type = 0;
 
-
   if (!udpsink_type) {
     static const GTypeInfo udpsink_info = {
       sizeof(GstUDPSinkClass),
-      NULL,
+      gst_udpsink_base_init,
       NULL,
       (GClassInitFunc)gst_udpsink_class_init,
       NULL,
@@ -111,6 +108,14 @@ gst_udpsink_get_type (void)
   return udpsink_type;
 }
 
+static void
+gst_udpsink_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_set_details (element_class, &gst_udpsink_details);
+}
+
 static void
 gst_udpsink_class_init (GstUDPSink *klass)
 {
@@ -145,7 +150,7 @@ gst_udpsink_class_init (GstUDPSink *klass)
 
 
 static GstPadLinkReturn
-gst_udpsink_sinkconnect (GstPad *pad, GstCaps *caps)
+gst_udpsink_sink_link (GstPad *pad, const GstCaps *caps)
 {
   GstUDPSink *udpsink;
   struct sockaddr_in serv_addr;
@@ -246,7 +251,7 @@ gst_udpsink_init (GstUDPSink *udpsink)
   udpsink->sinkpad = gst_pad_new ("sink", GST_PAD_SINK);
   gst_element_add_pad (GST_ELEMENT (udpsink), udpsink->sinkpad);
   gst_pad_set_chain_function (udpsink->sinkpad, gst_udpsink_chain);
-  gst_pad_set_link_function (udpsink->sinkpad, gst_udpsink_sinkconnect);
+  gst_pad_set_link_function (udpsink->sinkpad, gst_udpsink_sink_link);
 
   udpsink->host = g_strdup (UDP_DEFAULT_HOST);
   udpsink->port = UDP_DEFAULT_PORT;
@@ -269,12 +274,8 @@ gst_udpsink_chain (GstPad *pad, GstData *_data)
 
   udpsink = GST_UDPSINK (GST_OBJECT_PARENT (pad));
   
-  if (udpsink->clock) {
-    GstClockID id = gst_clock_new_single_shot_id (udpsink->clock, GST_BUFFER_TIMESTAMP (buf));
-
-    GST_DEBUG ("udpsink: clock wait: %" G_GUINT64_FORMAT "\n", GST_BUFFER_TIMESTAMP (buf));
-    gst_element_clock_wait (GST_ELEMENT (udpsink), id, NULL);
-    gst_clock_id_free (id);
+  if (udpsink->clock && GST_BUFFER_TIMESTAMP_IS_VALID (buf)) {
+    gst_element_wait (GST_ELEMENT (udpsink), GST_BUFFER_TIMESTAMP (buf));
   }
   
   tolen = sizeof(udpsink->theiraddr);
@@ -375,7 +376,7 @@ gst_udpsink_init_send (GstUDPSink *sink)
   struct in_addr addr;
   guint bc_val;
 
-  bzero (&sink->theiraddr, sizeof (sink->theiraddr));
+  memset (&sink->theiraddr, 0, sizeof (sink->theiraddr));
   sink->theiraddr.sin_family = AF_INET;         /* host byte order */
   sink->theiraddr.sin_port = htons (sink->port); /* short, network byte order */
 
