@@ -234,9 +234,17 @@ gst_udpsrc_get (GstPad *pad)
       buf[ret] = '\0';
       doc = xmlParseMemory(buf, ret);
       caps = gst_caps_load_thyself(doc->xmlRootNode);
+      if (caps == NULL) {
+        return NULL;
+      }
       
       /* foward the connect, we don't signal back the result here... */
-      gst_pad_proxy_link (udpsrc->srcpad, caps);
+      if (gst_caps_is_fixed (caps)) {
+        gst_pad_try_set_caps (udpsrc->srcpad, caps);
+      } else {
+        GST_ERROR ("caps %" GST_PTR_FORMAT, caps);
+        GST_ELEMENT_ERROR (udpsrc, CORE, NEGOTIATION, (NULL), ("Got unfixed caps from peer"));
+      }
 
 #endif
       g_free (buf);
@@ -353,7 +361,7 @@ gst_udpsrc_init_receive (GstUDPSrc *src)
 {
   guint bc_val;
   gint reuse=1;
-  bzero (&src->myaddr, sizeof (src->myaddr));
+  memset (&src->myaddr, 0, sizeof (src->myaddr));
   src->myaddr.sin_family = AF_INET;           /* host byte order */
   src->myaddr.sin_port = htons (src->port);   /* short, network byte order */
   src->myaddr.sin_addr.s_addr = INADDR_ANY;
