@@ -26,6 +26,9 @@
 /*#define DEBUG_ENABLED */
 #include <gstvideotestsrc.h>
 #include <videotestsrc.h>
+#ifdef HAVE_LIBOIL
+#include <liboil/liboil.h>
+#endif
 
 #include <string.h>
 #include <stdlib.h>
@@ -64,39 +67,6 @@ random_chars (unsigned char *dest, int nbytes)
 }
 #endif
 
-static void
-memset_str2 (unsigned char *dest, unsigned char val, int n)
-{
-  int i;
-
-  for (i = 0; i < n; i++) {
-    *dest = val;
-    dest += 2;
-  }
-}
-
-static void
-memset_str3 (unsigned char *dest, unsigned char val, int n)
-{
-  int i;
-
-  for (i = 0; i < n; i++) {
-    *dest = val;
-    dest += 3;
-  }
-}
-
-static void
-memset_str4 (unsigned char *dest, unsigned char val, int n)
-{
-  int i;
-
-  for (i = 0; i < n; i++) {
-    *dest = val;
-    dest += 4;
-  }
-}
-
 #if 0
 static void
 paint_rect_random (unsigned char *dest, int stride, int x, int y, int w, int h)
@@ -768,6 +738,17 @@ paint_setup_YVYU (paintinfo * p, char *dest)
   p->endptr = dest + p->ystride * p->height;
 }
 
+#ifndef HAVE_LIBOIL
+void splat_u8 (guint8 *dest, int dstr, guint8 val, int n)
+{
+  int i;
+  for(i=0;i<n;i++){
+    *dest = val;
+    dest += dstr;
+  }
+}
+#endif
+
 static void
 paint_hline_YUY2 (paintinfo * p, int x, int y, int w)
 {
@@ -776,9 +757,9 @@ paint_hline_YUY2 (paintinfo * p, int x, int y, int w)
   int offset;
 
   offset = y * p->ystride;
-  memset_str2 (p->yp + offset + x * 2, p->color->Y, w);
-  memset_str4 (p->up + offset + x1 * 4, p->color->U, x2 - x1);
-  memset_str4 (p->vp + offset + x1 * 4, p->color->V, x2 - x1);
+  splat_u8 (p->yp + offset + x * 2, 2, p->color->Y, w);
+  splat_u8 (p->up + offset + x1 * 4, 4, p->color->U, x2 - x1);
+  splat_u8 (p->vp + offset + x1 * 4, 4, p->color->V, x2 - x1);
 }
 
 static void
@@ -798,9 +779,9 @@ paint_hline_IYU2 (paintinfo * p, int x, int y, int w)
   int offset;
 
   offset = y * p->ystride;
-  memset_str3 (p->yp + offset + x * 3, p->color->Y, w);
-  memset_str3 (p->up + offset + x * 3, p->color->U, w);
-  memset_str3 (p->vp + offset + x * 3, p->color->V, w);
+  splat_u8 (p->yp + offset + x * 3, 3, p->color->Y, w);
+  splat_u8 (p->up + offset + x * 3, 3, p->color->U, w);
+  splat_u8 (p->vp + offset + x * 3, 3, p->color->V, w);
 }
 
 static void
@@ -972,9 +953,9 @@ paint_hline_str4 (paintinfo * p, int x, int y, int w)
 {
   int offset = y * p->ystride;
 
-  memset_str4 (p->yp + offset + x * 4, p->color->R, w);
-  memset_str4 (p->up + offset + x * 4, p->color->G, w);
-  memset_str4 (p->vp + offset + x * 4, p->color->B, w);
+  splat_u8 (p->yp + offset + x * 4, 4, p->color->R, w);
+  splat_u8 (p->up + offset + x * 4, 4, p->color->G, w);
+  splat_u8 (p->vp + offset + x * 4, 4, p->color->B, w);
 }
 
 static void
@@ -982,9 +963,9 @@ paint_hline_str3 (paintinfo * p, int x, int y, int w)
 {
   int offset = y * p->ystride;
 
-  memset_str3 (p->yp + offset + x * 3, p->color->R, w);
-  memset_str3 (p->up + offset + x * 3, p->color->G, w);
-  memset_str3 (p->vp + offset + x * 3, p->color->B, w);
+  splat_u8 (p->yp + offset + x * 3, 3, p->color->R, w);
+  splat_u8 (p->up + offset + x * 3, 3, p->color->G, w);
+  splat_u8 (p->vp + offset + x * 3, 3, p->color->B, w);
 }
 
 static void
@@ -1005,11 +986,11 @@ paint_hline_RGB565 (paintinfo * p, int x, int y, int w)
   b = ((p->color->G<<3)&0xe0) | (p->color->B>>3);
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
-  memset_str2 (p->yp + offset + x * 2 + 0, b, w);
-  memset_str2 (p->yp + offset + x * 2 + 1, a, w);
+  splat_u8 (p->yp + offset + x * 2 + 0, 2, b, w);
+  splat_u8 (p->yp + offset + x * 2 + 1, 2, a, w);
 #else
-  memset_str2 (p->yp + offset + x * 2 + 0, a, w);
-  memset_str2 (p->yp + offset + x * 2 + 1, b, w);
+  splat_u8 (p->yp + offset + x * 2 + 0, 2, a, w);
+  splat_u8 (p->yp + offset + x * 2 + 1, 2, b, w);
 #endif
 }
 
@@ -1031,11 +1012,11 @@ paint_hline_xRGB1555 (paintinfo * p, int x, int y, int w)
   b = ((p->color->G<<2)&0xe0) | (p->color->B>>3);
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
-  memset_str2 (p->yp + offset + x * 2 + 0, b, w);
-  memset_str2 (p->yp + offset + x * 2 + 1, a, w);
+  splat_u8 (p->yp + offset + x * 2 + 0, 2, b, w);
+  splat_u8 (p->yp + offset + x * 2 + 1, 2, a, w);
 #else
-  memset_str2 (p->yp + offset + x * 2 + 0, a, w);
-  memset_str2 (p->yp + offset + x * 2 + 1, b, w);
+  splat_u8 (p->yp + offset + x * 2 + 0, 2, a, w);
+  splat_u8 (p->yp + offset + x * 2 + 1, 2, b, w);
 #endif
 }
 
