@@ -18,27 +18,20 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 #include <config.h>
 #endif
 
 #include <string.h>
 #include <sys/time.h>
 #include "v4lsrc_calls.h"
-#include <gst/video/video.h>
 
 /* elementfactory information */
-static GstElementDetails gst_v4lsrc_details = {
+static GstElementDetails gst_v4lsrc_details = GST_ELEMENT_DETAILS (
   "Video (video4linux/raw) Source",
   "Source/Video",
-  "LGPL",
   "Reads raw frames from a video4linux (BT8x8) device",
-  VERSION,
-  "Ronald Bultje <rbultje@ronald.bitfreak.net>",
-  "(C) 2001",
-};
+  "Ronald Bultje <rbultje@ronald.bitfreak.net>"
+);
 
 /* V4lSrc signals and args */
 enum {
@@ -57,21 +50,37 @@ enum {
   ARG_USE_FIXED_FPS
 };
 
+GST_FORMATS_FUNCTION (GstPad *, gst_v4lsrc_get_formats,
+		      GST_FORMAT_TIME, GST_FORMAT_DEFAULT);
+GST_QUERY_TYPE_FUNCTION (GstPad *, gst_v4lsrc_get_query_types,
+			 GST_QUERY_POSITION);
 
 /* init functions */
+static void		     gst_v4lsrc_base_init    (gpointer g_class);
 static void                  gst_v4lsrc_class_init   (GstV4lSrcClass *klass);
 static void                  gst_v4lsrc_init         (GstV4lSrc      *v4lsrc);
 
-/* pad/buffer functions */
-static gboolean              gst_v4lsrc_srcconvert   (GstPad         *pad,
+/* parent class virtual functions */
+static void                  gst_v4lsrc_open         (GstElement     *element,
+						      const gchar    *device);
+static void                  gst_v4lsrc_close        (GstElement     *element,
+                                                      const gchar    *device);
+
+/* pad/info functions */
+static gboolean              gst_v4lsrc_src_convert  (GstPad         *pad,
                                                       GstFormat      src_format,
                                                       gint64         src_value,
                                                       GstFormat      *dest_format,
                                                       gint64         *dest_value);
+static gboolean              gst_v4lsrc_src_query    (GstPad         *pad,
+                                                      GstQueryType   type, 
+                                                      GstFormat      *format,
+                                                      gint64         *value);
+
+/* buffer functions */
 static GstPadLinkReturn      gst_v4lsrc_srcconnect   (GstPad         *pad,
-                                                      GstCaps        *caps);
-static GstCaps*	             gst_v4lsrc_getcaps      (GstPad         *pad,
-                                                      GstCaps        *caps);
+                                                      const GstCaps        *caps);
+static GstCaps*	             gst_v4lsrc_getcaps      (GstPad         *pad);
 static GstData*            gst_v4lsrc_get          (GstPad         *pad);
 
 /* get/set params */
@@ -87,20 +96,12 @@ static void                  gst_v4lsrc_get_property (GObject        *object,
 /* state handling */
 static GstElementStateReturn gst_v4lsrc_change_state (GstElement     *element);
 
-/* bufferpool functions */
-static GstBuffer*            gst_v4lsrc_buffer_new   (GstBufferPool  *pool,
-                                                      guint64        offset,
-                                                      guint          size,
-                                                      gpointer       user_data);
-static void                  gst_v4lsrc_buffer_free  (GstBufferPool  *pool,
-						      GstBuffer      *buf,
-						      gpointer       user_data);
-
 /* set_clock function for a/V sync */
 static void                  gst_v4lsrc_set_clock    (GstElement     *element,
                                                       GstClock       *clock);
 
-static GstPadTemplate *src_template = NULL;
+/* requeue buffer if it's back available */
+static void                  gst_v4lsrc_buffer_free  (GstBuffer      *buffer);
 
 static GstElementClass *parent_class = NULL;
 static guint gst_v4lsrc_signals[LAST_SIGNAL] = { 0 };
@@ -114,7 +115,7 @@ gst_v4lsrc_get_type (void)
   if (!v4lsrc_type) {
     static const GTypeInfo v4lsrc_info = {
       sizeof(GstV4lSrcClass),
-      NULL,
+      gst_v4lsrc_base_init,
       NULL,
       (GClassInitFunc)gst_v4lsrc_class_init,
       NULL,
@@ -129,15 +130,33 @@ gst_v4lsrc_get_type (void)
   return v4lsrc_type;
 }
 
+static void
+gst_v4lsrc_base_init (gpointer g_class)
+{
+  GstPadTemplate *src_template;
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
+  
+  gst_element_class_set_details (gstelement_class, &gst_v4lsrc_details);
+
+  src_template = gst_pad_template_new (
+		  "src",
+                  GST_PAD_SRC,
+  		  GST_PAD_ALWAYS,
+		  NULL);
+
+  gst_element_class_add_pad_template (gstelement_class, src_template);
+}
 
 static void
 gst_v4lsrc_class_init (GstV4lSrcClass *klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
+  GstV4lElementClass *v4lelement_class;
 
   gobject_class = (GObjectClass*)klass;
   gstelement_class = (GstElementClass*)klass;
+  v4lelement_class = (GstV4lElementClass *) klass;
 
   parent_class = g_type_class_ref(GST_TYPE_V4LELEMENT);
 
@@ -155,17 +174,17 @@ gst_v4lsrc_class_init (GstV4lSrcClass *klass)
 
   /* signals */
   gst_v4lsrc_signals[SIGNAL_FRAME_CAPTURE] =
-    g_signal_new("frame_capture", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
+    g_signal_new("frame-capture", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
                  G_STRUCT_OFFSET(GstV4lSrcClass, frame_capture),
                  NULL, NULL, g_cclosure_marshal_VOID__VOID,
                  G_TYPE_NONE, 0);
   gst_v4lsrc_signals[SIGNAL_FRAME_DROP] =
-    g_signal_new("frame_drop", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
+    g_signal_new("frame-drop", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
                  G_STRUCT_OFFSET(GstV4lSrcClass, frame_drop),
                  NULL, NULL, g_cclosure_marshal_VOID__VOID,
                  G_TYPE_NONE, 0);
   gst_v4lsrc_signals[SIGNAL_FRAME_INSERT] =
-    g_signal_new("frame_insert", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
+    g_signal_new("frame-insert", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
                  G_STRUCT_OFFSET(GstV4lSrcClass, frame_insert),
                  NULL, NULL, g_cclosure_marshal_VOID__VOID,
                  G_TYPE_NONE, 0);
@@ -176,39 +195,83 @@ gst_v4lsrc_class_init (GstV4lSrcClass *klass)
   gstelement_class->change_state = gst_v4lsrc_change_state;
 
   gstelement_class->set_clock = gst_v4lsrc_set_clock;
+
+  v4lelement_class->open  = gst_v4lsrc_open;
+  v4lelement_class->close = gst_v4lsrc_close;
 }
 
 
 static void
 gst_v4lsrc_init (GstV4lSrc *v4lsrc)
 {
+  GstElementClass *klass = GST_ELEMENT_GET_CLASS (v4lsrc);
+
   GST_FLAG_SET(GST_ELEMENT(v4lsrc), GST_ELEMENT_THREAD_SUGGESTED);
 
-  v4lsrc->srcpad = gst_pad_new_from_template (src_template, "src");
+  v4lsrc->srcpad = gst_pad_new_from_template (
+	gst_element_class_get_pad_template (klass, "src"), "src");
   gst_element_add_pad(GST_ELEMENT(v4lsrc), v4lsrc->srcpad);
 
   gst_pad_set_get_function (v4lsrc->srcpad, gst_v4lsrc_get);
   gst_pad_set_getcaps_function (v4lsrc->srcpad, gst_v4lsrc_getcaps);
   gst_pad_set_link_function (v4lsrc->srcpad, gst_v4lsrc_srcconnect);
-  gst_pad_set_convert_function (v4lsrc->srcpad, gst_v4lsrc_srcconvert);
-
-  v4lsrc->bufferpool = gst_buffer_pool_new(
-		  NULL, 
-		  NULL,
-		  (GstBufferPoolBufferNewFunction)gst_v4lsrc_buffer_new,
-		  NULL,
-		  (GstBufferPoolBufferFreeFunction)gst_v4lsrc_buffer_free,
-		  v4lsrc);
+  gst_pad_set_convert_function (v4lsrc->srcpad, gst_v4lsrc_src_convert);
+  gst_pad_set_formats_function (v4lsrc->srcpad, gst_v4lsrc_get_formats);
+  gst_pad_set_query_function (v4lsrc->srcpad, gst_v4lsrc_src_query);
+  gst_pad_set_query_type_function (v4lsrc->srcpad, gst_v4lsrc_get_query_types);
 
   v4lsrc->buffer_size = 0;
 
   /* no clock */
   v4lsrc->clock = NULL;
 
+  /* no colourspaces */
+  v4lsrc->colourspaces = NULL;
+
   /* fps */
   v4lsrc->use_fixed_fps = TRUE;
+
+  v4lsrc->is_capturing = FALSE;
 }
 
+static void
+gst_v4lsrc_open (GstElement  *element,
+                 const gchar *device)
+{
+  GstV4lSrc *v4lsrc = GST_V4LSRC (element);
+  int palette[] = {
+    VIDEO_PALETTE_YUV422,
+    VIDEO_PALETTE_YUV420P,
+    VIDEO_PALETTE_UYVY,
+    VIDEO_PALETTE_YUV411P,
+    VIDEO_PALETTE_YUV422P,
+    VIDEO_PALETTE_YUV410P,
+    VIDEO_PALETTE_YUV411,
+    VIDEO_PALETTE_RGB555,
+    VIDEO_PALETTE_RGB565,
+    VIDEO_PALETTE_RGB24,
+    VIDEO_PALETTE_RGB32,
+    -1
+  }, i;
+
+  for (i = 0; palette[i] != -1; i++) {
+    /* try palette out */
+    if (!gst_v4lsrc_try_palette(v4lsrc, palette[i]))
+      continue;
+    v4lsrc->colourspaces = g_list_append (v4lsrc->colourspaces,
+					  GINT_TO_POINTER (palette[i]));
+  }
+}
+
+static void
+gst_v4lsrc_close (GstElement  *element,
+                  const gchar *device)
+{
+  GstV4lSrc *v4lsrc = GST_V4LSRC (element);
+
+  g_list_free (v4lsrc->colourspaces);
+  v4lsrc->colourspaces = NULL;
+}
 
 static gfloat
 gst_v4lsrc_get_fps (GstV4lSrc *v4lsrc)
@@ -240,13 +303,12 @@ gst_v4lsrc_get_fps (GstV4lSrc *v4lsrc)
   return fps;
 }
 
-
 static gboolean
-gst_v4lsrc_srcconvert (GstPad    *pad,
-                       GstFormat  src_format,
-                       gint64     src_value,
-                       GstFormat *dest_format,
-                       gint64    *dest_value)
+gst_v4lsrc_src_convert (GstPad    *pad,
+                        GstFormat  src_format,
+                        gint64     src_value,
+                        GstFormat *dest_format,
+                        gint64    *dest_value)
 {
   GstV4lSrc *v4lsrc;
   gdouble fps;
@@ -284,11 +346,43 @@ gst_v4lsrc_srcconvert (GstPad    *pad,
   return TRUE;
 }
 
+static gboolean
+gst_v4lsrc_src_query (GstPad      *pad,
+                      GstQueryType type, 
+                      GstFormat   *format,
+                      gint64      *value)
+{
+  GstV4lSrc *v4lsrc = GST_V4LSRC (gst_pad_get_parent (pad));
+  gboolean res = TRUE;
+  gdouble fps;
+
+  if ((fps = gst_v4lsrc_get_fps(v4lsrc)) == 0)
+    return FALSE;
+
+  switch (type) {
+    case GST_QUERY_POSITION:
+      switch (*format) {
+        case GST_FORMAT_TIME:
+          *value = v4lsrc->handled * GST_SECOND / fps;
+          break;
+        case GST_FORMAT_DEFAULT:
+          *value = v4lsrc->handled;
+          break;
+        default:
+          res = FALSE;
+          break;
+      }
+      break;
+    default:
+      res = FALSE;
+      break;
+  }
+
+  return res;
+}
+
 static GstCaps *
-gst_v4lsrc_palette_to_caps (int            palette,
-                            GstPropsEntry *width,
-                            GstPropsEntry *height,
-                            GstPropsEntry *fps)
+gst_v4lsrc_palette_to_caps (int palette)
 {
   guint32 fourcc;
   GstCaps *caps;
@@ -304,9 +398,18 @@ gst_v4lsrc_palette_to_caps (int            palette,
     case VIDEO_PALETTE_UYVY:
       fourcc = GST_MAKE_FOURCC('U','Y','V','Y');
       break;
+    case VIDEO_PALETTE_YUV411P:
+      fourcc = GST_MAKE_FOURCC('Y','4','1','B');
+      break;
     case VIDEO_PALETTE_YUV411:
       fourcc = GST_MAKE_FOURCC('Y','4','1','P');
       break;
+    case VIDEO_PALETTE_YUV422P:
+      fourcc = GST_MAKE_FOURCC('Y','4','2','B');
+      break;
+    case VIDEO_PALETTE_YUV410P:
+      fourcc = GST_MAKE_FOURCC('Y','U','V','9');
+      break;
     case VIDEO_PALETTE_RGB555:
     case VIDEO_PALETTE_RGB565:
     case VIDEO_PALETTE_RGB24:
@@ -318,248 +421,210 @@ gst_v4lsrc_palette_to_caps (int            palette,
   }
 
   if (fourcc == GST_MAKE_FOURCC('R','G','B',' ')) {
-    gint depth = 0, endianness = 0;
-    guint32 r_mask = 0, g_mask = 0, b_mask = 0;
-
     switch (palette) {
       case VIDEO_PALETTE_RGB555:
-        depth = 15; endianness = G_BYTE_ORDER;
-        r_mask = 0x7c00; g_mask = 0x03e0; b_mask = 0x001f;
+	caps = gst_caps_from_string ("video/x-raw-rgb, "
+	    "bpp = (int) 16, "
+	    "depth = (int) 15, "
+	    "endianness = (int) BYTE_ORDER, "
+	    "red_mask = 0x7c00, "
+	    "green_mask = 0x03e0, "
+	    "blue_mask = 0x001f");
         break;
       case VIDEO_PALETTE_RGB565:
-        depth = 16; endianness = G_BYTE_ORDER;
-        r_mask = 0xf800; g_mask = 0x07f0; b_mask = 0x001f;
+	caps = gst_caps_from_string ("video/x-raw-rgb, "
+	    "bpp = (int) 16, "
+	    "depth = (int) 16, "
+	    "endianness = (int) BYTE_ORDER, "
+	    "red_mask = 0xf800, "
+	    "green_mask = 0x07f0, "
+	    "blue_mask = 0x001f");
         break;
       case VIDEO_PALETTE_RGB24:
-        depth = 24; endianness = G_BIG_ENDIAN;
-        r_mask = R_MASK_24; g_mask = G_MASK_24; b_mask = B_MASK_24;
+	caps = gst_caps_from_string ("video/x-raw-rgb, "
+	    "bpp = (int) 24, "
+	    "depth = (int) 24, "
+	    "endianness = (int) BIG_ENDIAN, "
+	    "red_mask = 0xFF0000, "
+	    "green_mask = 0x00FF00, "
+	    "blue_mask = 0x0000FF");
         break;
       case VIDEO_PALETTE_RGB32:
-        depth = 32; endianness = G_BIG_ENDIAN;
-        r_mask = R_MASK_32; g_mask = G_MASK_32; b_mask = B_MASK_32;
+	caps = gst_caps_from_string ("video/x-raw-rgb, "
+	    "bpp = (int) 24, "
+	    "depth = (int) 32, "
+	    "endianness = (int) BIG_ENDIAN, "
+	    "red_mask = 0xFF000000, "
+	    "green_mask = 0x00FF0000, "
+	    "blue_mask = 0x0000FF00");
         break;
       default:
         g_assert_not_reached();
-        break;
+	return NULL;
     }
-
-    caps = GST_CAPS_NEW("v4lsrc_rgb_caps",
-                        "video/x-raw-rgb",
-                          "bpp",        GST_PROPS_INT((depth+1) & ~1),
-                          "depth",      GST_PROPS_INT(depth),
-                          "endianness", GST_PROPS_INT(G_BYTE_ORDER),
-                          "red_mask",   GST_PROPS_INT(r_mask),
-                          "green_mask", GST_PROPS_INT(g_mask),
-                          "blue_mask",  GST_PROPS_INT(b_mask),
-                          NULL);
   } else {
-    caps = GST_CAPS_NEW("v4lsrc_yuv_caps",
-                        "video/x-raw-yuv",
-                          "format",     GST_PROPS_FOURCC(fourcc),
-                          NULL);
+    caps = gst_caps_new_simple ("video/x-raw-yuv",
+	"format", GST_TYPE_FOURCC, fourcc, NULL);
   }
 
-  gst_props_add_entry(caps->properties, width);
-  gst_props_add_entry(caps->properties, height);
-  gst_props_add_entry(caps->properties, fps);
-
   return caps;
 }
 
-#define gst_v4lsrc_palette_to_caps_fixed(palette, width, height, fps) \
-  gst_v4lsrc_palette_to_caps(palette, \
-                             gst_props_entry_new("width", \
-                                                 GST_PROPS_INT(width)), \
-                             gst_props_entry_new("height", \
-                                                 GST_PROPS_INT(height)), \
-                             gst_props_entry_new("framerate", \
-                                                 GST_PROPS_FLOAT(fps)) \
-                            )
-#define gst_v4lsrc_palette_to_caps_range(palette, min_w, max_w, min_h, max_h) \
-  gst_v4lsrc_palette_to_caps(palette, \
-                             gst_props_entry_new("width", \
-                                                 GST_PROPS_INT_RANGE(min_w, \
-                                                                     max_w)), \
-                             gst_props_entry_new("height", \
-                                                 GST_PROPS_INT_RANGE(min_h, \
-                                                                     max_h)), \
-                             gst_props_entry_new("framerate", \
-                                                 GST_PROPS_FLOAT_RANGE(0., \
-                                                                 G_MAXFLOAT)) \
-                            )
-
-#define gst_caps_get_int_range(caps, name, min, max) \
-  gst_props_entry_get_int_range(gst_props_get_entry((caps)->properties, \
-                                                    name), \
-                                min, max)
 
 static GstPadLinkReturn
-gst_v4lsrc_srcconnect (GstPad  *pad,
-                       GstCaps *vscapslist)
+gst_v4lsrc_srcconnect (GstPad  *pad, const GstCaps *vscapslist)
 {
-  GstPadLinkReturn ret_val;
   GstV4lSrc *v4lsrc;
-  GstCaps *caps, *newcaps;
-  gint palette;
+  guint32 fourcc;
+  gint bpp, depth, w, h, palette = -1;
+  gdouble fps;
+  GstStructure *structure;
+  gboolean was_capturing;
 
   v4lsrc = GST_V4LSRC (gst_pad_get_parent (pad));
+  was_capturing = v4lsrc->is_capturing;
 
   /* in case the buffers are active (which means that we already
    * did capsnego before and didn't clean up), clean up anyways */
   if (GST_V4L_IS_ACTIVE(GST_V4LELEMENT(v4lsrc)))
   {
+    if (was_capturing) {
+      if (!gst_v4lsrc_capture_stop(v4lsrc))
+        return GST_PAD_LINK_REFUSED;
+    }
     if (!gst_v4lsrc_capture_deinit(v4lsrc))
       return GST_PAD_LINK_REFUSED;
-  }
-  else if (!GST_V4L_IS_OPEN(GST_V4LELEMENT(v4lsrc)))
-  {
+  } else if (!GST_V4L_IS_OPEN(GST_V4LELEMENT(v4lsrc))) {
     return GST_PAD_LINK_DELAYED;
   }
 
-  /* TODO: caps = gst_caps_normalize(capslist); */
-  for (caps = vscapslist ; caps != NULL ; caps = vscapslist = vscapslist->next)
+  structure = gst_caps_get_structure (vscapslist, 0);
+
+  if (!strcmp(gst_structure_get_name (structure), "video/x-raw-yuv"))
+    gst_structure_get_fourcc (structure, "format", &fourcc);
+  else
+    fourcc = GST_MAKE_FOURCC('R','G','B',' ');
+
+  gst_structure_get_int (structure, "width", &w);
+  gst_structure_get_int (structure, "height", &h);
+  gst_structure_get_double (structure, "framerate", &fps);
+
+  switch (fourcc)
   {
-    guint32 fourcc;
-    gint depth, w, h;
-
-    if (!strcmp(gst_caps_get_mime(caps), "video/x-raw-yuv"))
-      gst_caps_get_fourcc_int (caps, "format", &fourcc);
-    else
-      fourcc = GST_MAKE_FOURCC('R','G','B',' ');
-
-    if (gst_caps_has_property(caps, "width")) {
-      if (gst_caps_has_fixed_property(caps, "width")) {
-        gst_caps_get_int(caps, "width", &w);
-      } else {
-        int min;
-        gst_caps_get_int_range(caps, "width", &min, &w);
+    case GST_MAKE_FOURCC('I','4','2','0'):
+      palette = VIDEO_PALETTE_YUV420P;
+      v4lsrc->buffer_size = ((w+1)&~1) * ((h+1)&~1) * 1.5;
+      break;
+    case GST_MAKE_FOURCC('Y','U','Y','2'):
+      palette = VIDEO_PALETTE_YUV422;
+      v4lsrc->buffer_size = ((w+1)&~1) * h * 2;
+      break;
+    case GST_MAKE_FOURCC('U','Y','V','Y'):
+      palette = VIDEO_PALETTE_UYVY;
+      v4lsrc->buffer_size = ((w+1)&~1) * h * 2;
+      break;
+    case GST_MAKE_FOURCC('Y','4','1','B'):
+      palette = VIDEO_PALETTE_YUV411P;
+      v4lsrc->buffer_size = ((w+3)&~3) * h * 1.5;
+      break;
+    case GST_MAKE_FOURCC('Y','4','1','P'):
+      palette = VIDEO_PALETTE_YUV411;
+      v4lsrc->buffer_size = ((w+3)&~3) * h * 1.5;
+      break;
+    case GST_MAKE_FOURCC('Y','U','V','9'):
+      palette = VIDEO_PALETTE_YUV410P;
+      v4lsrc->buffer_size = ((w+3)&~3) * ((h+3)&~3) * 1.125;
+      break;
+    case GST_MAKE_FOURCC('Y','4','2','B'):
+      palette = VIDEO_PALETTE_YUV422P;
+      v4lsrc->buffer_size = ((w+1)&~1) * h * 2;
+      break;
+    case GST_MAKE_FOURCC('R','G','B',' '):
+      gst_structure_get_int (structure, "depth", &depth);
+      switch (depth)
+      {
+	case 15:
+	  palette = VIDEO_PALETTE_RGB555;
+	  v4lsrc->buffer_size = w * h * 2;
+          break;
+	case 16:
+	  palette = VIDEO_PALETTE_RGB565;
+	  v4lsrc->buffer_size = w * h * 2;
+          break;
+	case 24:
+          gst_structure_get_int (structure, "bpp", &bpp);
+          switch (bpp) {
+            case 24:
+	      palette = VIDEO_PALETTE_RGB24;
+	      v4lsrc->buffer_size = w * h * 3;
+              break;
+	    case 32:
+	      palette = VIDEO_PALETTE_RGB32;
+	      v4lsrc->buffer_size = w * h * 4;
+              break;
+            default:
+              break;
+          }
+          break;
+	default:
+          break;
       }
-    }
-    if (gst_caps_has_property(caps, "height")) {
-      if (gst_caps_has_fixed_property(caps, "height")) {
-        gst_caps_get_int(caps, "height", &h);
-      } else {
-        int min;
-        gst_caps_get_int_range(caps, "height", &min, &h);
-      }
-    }
-
-    switch (fourcc)
-    {
-      case GST_MAKE_FOURCC('I','4','2','0'):
-      case GST_MAKE_FOURCC('I','Y','U','V'):
-        palette = VIDEO_PALETTE_YUV420P;
-        v4lsrc->buffer_size = ((w+1)&~1) * ((h+1)&~1) * 1.5;
-        goto try_caps;
-      case GST_MAKE_FOURCC('Y','U','Y','2'):
-        palette = VIDEO_PALETTE_YUV422;
-        v4lsrc->buffer_size = ((w+1)&~1) * h * 2;
-        goto try_caps;
-      case GST_MAKE_FOURCC('U','Y','V','Y'):
-        palette = VIDEO_PALETTE_UYVY;
-        v4lsrc->buffer_size = ((w+1)&~1) * h * 2;
-        goto try_caps;
-      case GST_MAKE_FOURCC('Y','4','1','P'):
-        palette = VIDEO_PALETTE_YUV411;
-        v4lsrc->buffer_size = ((w+3)&~3) * h * 1.5;
-        goto try_caps;
-      case GST_MAKE_FOURCC('R','G','B',' '):
-        depth = gst_caps_get_int (caps, "depth", &depth);
-        switch (depth)
-        {
-          case 15:
-            palette = VIDEO_PALETTE_RGB555;
-            v4lsrc->buffer_size = w * h * 2;
-            goto try_caps;
-          case 16:
-            palette = VIDEO_PALETTE_RGB565;
-            v4lsrc->buffer_size = w * h * 2;
-            goto try_caps;
-          case 24:
-            palette = VIDEO_PALETTE_RGB24;
-            v4lsrc->buffer_size = w * h * 3;
-            goto try_caps;
-          case 32:
-            palette = VIDEO_PALETTE_RGB32;
-            v4lsrc->buffer_size = w * h * 4;
-            goto try_caps;
-          default:
-            goto try_next;
-        }
-      default:
-        goto try_next;
-    }
-
-  /* if this caps wasn't useful, try the next one */
-  try_next:
-    continue;
-
-  /* if this caps was useful, try it out */
-  try_caps:
-    /* try the current 'palette' out on the video device */
-    if (!gst_v4lsrc_try_palette(v4lsrc, palette))
-      continue;
-
-    /* try to connect the pad/caps with the actual width/height */
-    newcaps = gst_v4lsrc_palette_to_caps_fixed(palette, w, h,
-				gst_v4lsrc_get_fps(v4lsrc));
-
-    gst_caps_debug (newcaps, "new caps to set on v4lsrc's src pad");
-
-    if ((ret_val = gst_pad_try_set_caps(v4lsrc->srcpad, newcaps)) == GST_PAD_LINK_REFUSED)
-      continue;
-    else if (ret_val == GST_PAD_LINK_DELAYED)
-      return GST_PAD_LINK_DELAYED;
-
-    if (!gst_v4lsrc_set_capture(v4lsrc, w, h, palette))
-      return GST_PAD_LINK_REFUSED;
-
-    if (!gst_v4lsrc_capture_init(v4lsrc))
-      return GST_PAD_LINK_REFUSED;
-
-    return GST_PAD_LINK_DONE;
+      break;
+    default:
+      break;
   }
 
-  /* still nothing - no good caps */
-  return GST_PAD_LINK_REFUSED;
+  if (palette == -1)
+    return GST_PAD_LINK_REFUSED;
+
+  if (!gst_v4lsrc_set_capture(v4lsrc, w, h, palette))
+    return GST_PAD_LINK_REFUSED;
+
+  if (!gst_v4lsrc_capture_init(v4lsrc))
+    return GST_PAD_LINK_REFUSED;
+
+  if (was_capturing) {
+    if (!gst_v4lsrc_capture_start(v4lsrc))
+      return GST_PAD_LINK_REFUSED;
+  }
+
+  return GST_PAD_LINK_OK;
 }
 
 
 static GstCaps *
-gst_v4lsrc_getcaps (GstPad  *pad,
-                    GstCaps *caps)
+gst_v4lsrc_getcaps (GstPad  *pad)
 {
-  GstCaps *list = NULL;
+  GstCaps *list;
   GstV4lSrc *v4lsrc = GST_V4LSRC(gst_pad_get_parent(pad));
-  int palette[] = {
-    VIDEO_PALETTE_YUV422,
-    VIDEO_PALETTE_YUV420P,
-    VIDEO_PALETTE_UYVY,
-    VIDEO_PALETTE_YUV411P,
-    VIDEO_PALETTE_RGB555,
-    VIDEO_PALETTE_RGB565,
-    VIDEO_PALETTE_RGB24,
-    VIDEO_PALETTE_RGB32,
-  }, i;
   struct video_capability *vcap = &GST_V4LELEMENT(v4lsrc)->vcap;
+  GList *item;
 
   if (!GST_V4L_IS_OPEN(GST_V4LELEMENT(v4lsrc))) {
-    return NULL;
+    return gst_caps_new_any ();
   }
 
-  for (i = 0; i < 8; i++) {
+  list = gst_caps_new_empty();
+  for (item = v4lsrc->colourspaces; item != NULL; item = item->next) {
     GstCaps *one;
-    one = gst_v4lsrc_palette_to_caps_range(palette[i],
-                                           vcap->minwidth,  vcap->maxwidth,
-                                           vcap->minheight, vcap->maxheight);
-    list = gst_caps_append(list, one);
+
+    one = gst_v4lsrc_palette_to_caps(GPOINTER_TO_INT (item->data));
+    if (!one)
+      g_print ("Palette %d gave no caps\n",
+	       GPOINTER_TO_INT (item->data));
+    gst_caps_set_simple (one,
+	"width", GST_TYPE_INT_RANGE, vcap->minwidth,  vcap->maxwidth,
+	"height", GST_TYPE_INT_RANGE, vcap->minheight, vcap->maxheight,
+        "framerate", G_TYPE_DOUBLE, gst_v4lsrc_get_fps(v4lsrc),
+	NULL);
+    gst_caps_append(list, one);
   }
 
   return list;
 }
 
 
-static GstData*
+static GstData *
 gst_v4lsrc_get (GstPad *pad)
 {
   GstV4lSrc *v4lsrc;
@@ -575,14 +640,6 @@ gst_v4lsrc_get (GstPad *pad)
       (fps = gst_v4lsrc_get_fps(v4lsrc)) == 0)
     return NULL;
 
-  buf = gst_buffer_new_from_pool(v4lsrc->bufferpool, 0, 0);
-  if (!buf)
-  {
-    gst_element_error(GST_ELEMENT(v4lsrc),
-      "Failed to create a new GstBuffer");
-    return NULL;
-  }
-
   if (v4lsrc->need_writes > 0) {
     /* use last frame */
     num = v4lsrc->last_frame;
@@ -640,7 +697,12 @@ gst_v4lsrc_get (GstPad *pad)
     v4lsrc->use_num_times[num] = 1;
   }
 
+  buf = gst_buffer_new ();
+  GST_BUFFER_FREE_DATA_FUNC (buf) = gst_v4lsrc_buffer_free;
+  GST_BUFFER_PRIVATE (buf) = v4lsrc; /* hack to re-queue buffer on free */
+  GST_BUFFER_FLAG_SET (buf, GST_BUFFER_READONLY | GST_BUFFER_DONTFREE);
   GST_BUFFER_DATA(buf) = gst_v4lsrc_get_buffer(v4lsrc, num);
+  GST_BUFFER_MAXSIZE (buf) = v4lsrc->mbuf.size / v4lsrc->mbuf.frames;
   GST_BUFFER_SIZE(buf) = v4lsrc->buffer_size;
   if (v4lsrc->use_fixed_fps)
     GST_BUFFER_TIMESTAMP(buf) = v4lsrc->handled * GST_SECOND / fps;
@@ -766,6 +828,7 @@ gst_v4lsrc_change_state (GstElement *element)
 }
 
 
+#if 0
 static GstBuffer*
 gst_v4lsrc_buffer_new (GstBufferPool *pool,
 		       guint64 	     offset,
@@ -789,12 +852,12 @@ gst_v4lsrc_buffer_new (GstBufferPool *pool,
 
   return buffer;
 }
-
+#endif
 
 static void
-gst_v4lsrc_buffer_free (GstBufferPool *pool, GstBuffer *buf, gpointer user_data)
+gst_v4lsrc_buffer_free (GstBuffer *buf)
 {
-  GstV4lSrc *v4lsrc = GST_V4LSRC (user_data);
+  GstV4lSrc *v4lsrc = GST_V4LSRC (GST_BUFFER_PRIVATE (buf));
   int n;
 
   if (gst_element_get_state(GST_ELEMENT(v4lsrc)) != GST_STATE_PLAYING)
@@ -811,11 +874,8 @@ gst_v4lsrc_buffer_free (GstBufferPool *pool, GstBuffer *buf, gpointer user_data)
     }
 
   if (n == v4lsrc->mbuf.frames)
-    gst_element_error(GST_ELEMENT(v4lsrc),
-      "Couldn\'t find the buffer");
-
-  /* free struct */
-  gst_buffer_default_free(buf);
+    GST_ELEMENT_ERROR (v4lsrc, RESOURCE, TOO_LAZY, (NULL),
+      ("Couldn\'t find the buffer"));
 }
 
 
@@ -825,27 +885,3 @@ gst_v4lsrc_set_clock (GstElement *element,
 {
   GST_V4LSRC(element)->clock = clock;
 }
-
-
-gboolean
-gst_v4lsrc_factory_init (GstPlugin *plugin)
-{
-  GstElementFactory *factory;
-
-  /* create an elementfactory for the v4lsrc */
-  factory = gst_element_factory_new("v4lsrc",GST_TYPE_V4LSRC,
-                                   &gst_v4lsrc_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-
-  src_template = gst_pad_template_new (
-		  "src",
-                  GST_PAD_SRC,
-  		  GST_PAD_ALWAYS,
-		  NULL);
-
-  gst_element_factory_add_pad_template (factory, src_template);
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  return TRUE;
-}
